'/*=========================================
' DotNetBips.ApplicationBlocks.Data
' Version 1.0
' Date: 19 October, 2002
' Author: Bipin Joshi (webmaster@dotnetbips.com)
'=========================================*/

Imports System.Data

Public NotInheritable Class DataHelper

    Private Sub New()

    End Sub

    'Common Functions

    Private Shared Sub AttachParameters(ByVal command As IDbCommand, ByVal commandParameters() As IDataParameter)
        Dim p As IDataParameter
        For Each p In commandParameters
            If p.Direction = ParameterDirection.InputOutput And p.Value Is Nothing Then
                p.Value = Nothing
            End If
            command.Parameters.Add(p)
        Next p
    End Sub

    Private Shared Sub AssignParameterValues(ByVal commandParameters() As IDataParameter, ByVal parameterValues() As Object)
        Dim i As Short
        Dim j As Short
        If (commandParameters Is Nothing) And (parameterValues Is Nothing) Then
            Return
        End If
        If commandParameters.Length <> parameterValues.Length Then
            Throw New ArgumentException("Parameter count does not match Parameter Value count.")
        End If
        j = commandParameters.Length - 1
        For i = 0 To j
            commandParameters(i).Value = parameterValues(i)
        Next
    End Sub

    Private Shared Sub PrepareCommand(ByVal command As IDbCommand, ByVal connection As IDbConnection, ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal commandParameters() As IDataParameter)
        If connection.State <> ConnectionState.Open Then
            connection.Open()
        End If
        command.Connection = connection
        command.CommandText = commandText
        If Not (transaction Is Nothing) Then
            command.Transaction = transaction
        End If
        command.CommandType = commandType
        If Not (commandParameters Is Nothing) Then
            AttachParameters(command, commandParameters)
        End If
        Return
    End Sub

    'ExecuteNonQuery Overloads

    Public Overloads Shared Function ExecuteNonQuery(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String) As Integer
        Return ExecuteNonQuery(connectionString, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteNonQuery(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Integer
        Dim cn As IDbConnection = DataFactory.CreateConnection()
        cn.ConnectionString = connectionString
        cn.ConnectionString = connectionString
        Try
            cn.Open()
            Return ExecuteNonQuery(cn, commandType, commandText, commandParameters)
        Finally
            cn.Dispose()
        End Try
    End Function

    Public Overloads Shared Function ExecuteNonQuery(ByVal connection As IDbConnection, _
                                                     ByVal commandType As CommandType, _
                                                     ByVal commandText As String) As Integer
        Return ExecuteNonQuery(connection, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteNonQuery(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Integer
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim retval As Integer
        PrepareCommand(cmd, connection, CType(Nothing, IDbTransaction), commandType, commandText, commandParameters)
        retval = cmd.ExecuteNonQuery()
        cmd.Parameters.Clear()
        Return retval
    End Function

    Public Overloads Shared Function ExecuteNonQuery(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String) As Integer
        Return ExecuteNonQuery(transaction, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteNonQuery(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Integer
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim retval As Integer
        PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters)
        retval = cmd.ExecuteNonQuery()
        cmd.Parameters.Clear()
        Return retval
    End Function

    'ExecuteDataSet Overloads

    Public Overloads Shared Function ExecuteDataset(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String) As DataSet
        Return ExecuteDataset(connectionString, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteDataset(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As DataSet
        Dim cn As IDbConnection = DataFactory.CreateConnection()
        cn.ConnectionString = connectionString
        Try
            cn.Open()
            Return ExecuteDataset(cn, commandType, commandText, commandParameters)
        Finally
            cn.Dispose()
        End Try
    End Function

    Public Overloads Shared Function ExecuteDataset(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String) As DataSet
        Return ExecuteDataset(connection, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteDataset(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As DataSet
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim ds As New DataSet()
        Dim da As IDbDataAdapter
        PrepareCommand(cmd, connection, CType(Nothing, IDbTransaction), commandType, commandText, commandParameters)
        da = DataFactory.CreateDataAdapter()
        da.SelectCommand = cmd
        da.Fill(ds)
        cmd.Parameters.Clear()
        Return ds
    End Function

    Public Overloads Shared Function ExecuteDataset(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String) As DataSet
        Return ExecuteDataset(transaction, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteDataset(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As DataSet
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim ds As New DataSet()
        Dim da As IDbDataAdapter
        PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters)
        da = DataFactory.CreateDataAdapter()
        da.SelectCommand = cmd
        da.Fill(ds)
        cmd.Parameters.Clear()
        Return ds
    End Function

    Private Enum ConnectionOwnership
        Internal
        [External]
    End Enum

    'ExecuteReader Overloads

    Public Overloads Shared Function ExecuteReader(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String) As IDataReader
        Return ExecuteReader(connectionString, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteReader(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As IDataReader
        Dim cn As IDbConnection = DataFactory.CreateConnection()
        cn.ConnectionString = connectionString
        cn.Open()
        Try
            Return ExecuteReader(cn, CType(Nothing, IDbTransaction), commandType, commandText, commandParameters, ConnectionOwnership.Internal)
        Catch
            cn.Dispose()
        End Try
    End Function

    Public Overloads Shared Function ExecuteReader(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String) As IDataReader
        Return ExecuteReader(connection, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteReader(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As IDataReader
        Return ExecuteReader(connection, CType(Nothing, IDbTransaction), commandType, commandText, commandParameters, ConnectionOwnership.External)
    End Function

    Public Overloads Shared Function ExecuteReader(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String) As IDataReader
        Return ExecuteReader(transaction, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteReader(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As IDataReader
        Return ExecuteReader(transaction.Connection, transaction, commandType, commandText, commandParameters, ConnectionOwnership.External)
    End Function

    Private Overloads Shared Function ExecuteReader(ByVal connection As IDbConnection, ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal commandParameters() As IDataParameter, ByVal connectionOwnership As ConnectionOwnership) As IDataReader
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim dr As IDataReader
        PrepareCommand(cmd, connection, transaction, commandType, commandText, commandParameters)
        If connectionOwnership = connectionOwnership.External Then
            dr = cmd.ExecuteReader()
        Else
            dr = cmd.ExecuteReader(CommandBehavior.CloseConnection)
        End If
        cmd.Parameters.Clear()
        Return dr
    End Function


    'ExecuteScalar Overloads

    Public Overloads Shared Function ExecuteScalar(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String) As Object
        Return ExecuteScalar(connectionString, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteScalar(ByVal connectionString As String, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Object
        Dim cn As IDbConnection = DataFactory.CreateConnection()
        cn.ConnectionString = connectionString
        Try
            cn.Open()
            Return ExecuteScalar(cn, commandType, commandText, commandParameters)
        Finally
            cn.Dispose()
        End Try
    End Function

    Public Overloads Shared Function ExecuteScalar(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String) As Object
        Return ExecuteScalar(connection, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteScalar(ByVal connection As IDbConnection, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Object
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim retval As Object
        PrepareCommand(cmd, connection, CType(Nothing, IDbTransaction), commandType, commandText, commandParameters)
        retval = cmd.ExecuteScalar()
        cmd.Parameters.Clear()
        Return retval
    End Function

    Public Overloads Shared Function ExecuteScalar(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String) As Object
        Return ExecuteScalar(transaction, commandType, commandText, CType(Nothing, IDataParameter()))
    End Function

    Public Overloads Shared Function ExecuteScalar(ByVal transaction As IDbTransaction, ByVal commandType As CommandType, ByVal commandText As String, ByVal ParamArray commandParameters() As IDataParameter) As Object
        Dim cmd As IDbCommand = DataFactory.CreateCommand()
        Dim retval As Object
        PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters)
        retval = cmd.ExecuteScalar()
        cmd.Parameters.Clear()
        Return retval
    End Function

End Class


